using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;
using SushiBarBusinessLogic.BusinessLogics;
using SushiBarBusinessLogic.OfficePackage.Implements;
using SushiBarBusinessLogic.OfficePackage;
using SushiBarContracts.BusinessLogicsContracts;
using SushiBarContracts.StoragesContracts;
using SushiBarDatabaseImplement.Implements;
using SushiBarBusinessLogic;

namespace SushiBarView
{
    internal static class Program
    {
        private static ServiceProvider? _serviceProvider;
        public static ServiceProvider? ServiceProvider => _serviceProvider;
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();

            var services = new ServiceCollection();
            ConfigureServices(services);
            _serviceProvider = services.BuildServiceProvider();

            Application.Run(_serviceProvider.GetRequiredService<FormMain>());
        }

        private static void ConfigureServices(ServiceCollection services)
        {
            services.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                option.AddNLog("nlog.config");
            });
            services.AddTransient<IIngredientStorage, IngredientStorage>();
            services.AddTransient<IOrderStorage, OrderStorage>();
            services.AddTransient<ISushiStorage, SushiStorage>();
            services.AddTransient<IClientStorage, ClientStorage>();
            services.AddTransient<IImplementerStorage, ImplementerStorage>();

            services.AddTransient<IIngredientLogic, IngredientLogic>();
            services.AddTransient<IOrderLogic, OrderLogic>();
            services.AddTransient<ISushiLogic, SushiLogic>();
            services.AddTransient<IClientLogic, ClientLogic>();
            services.AddTransient<IReportLogic, ReportLogic>();
            services.AddTransient<IImplementerLogic, ImplementerLogic>();
            services.AddTransient<IWorkProcess, WorkModeling>();

            services.AddTransient<FormMain>();
            services.AddTransient<FormIngredient>();
            services.AddTransient<FormIngredients>();
            services.AddTransient<FormCreateOrder>();
            services.AddTransient<FormSushi>();
            services.AddTransient<FormSushiIngredients>();
            services.AddTransient<FormListSushi>();
            services.AddTransient<FormClients>();
            services.AddTransient<FormReportSushiIngredients>();
            services.AddTransient<FormReportOrders>(); 
            services.AddTransient<FormImplementers>();
            services.AddTransient<FormImplementer>();


            services.AddTransient<AbstractSaveToExcel, SaveToExcel>();
            services.AddTransient<AbstractSaveToWord, SaveToWord>();
            services.AddTransient<AbstractSaveToPdf, SaveToPdf>();
        }

    }
}
